import os
from colorthief import ColorThief
import colorsys

from PIL import Image


def average_color(image_file):
    # Open the image file using PIL
    with Image.open(image_file) as image:
        # Convert the image to RGBA format (if it's not already)
        # and get the width and height of the image
        image = image.convert("RGBA")
        width, height = image.size

        # Initialize variables for the average red, green, and blue values
        total_red = 0
        total_green = 0
        total_blue = 0
        total_pixels = 0

        # Loop through all the pixels in the image
        for x in range(width):
            for y in range(height):
                # Get the RGBA values for the pixel
                red, green, blue, alpha = image.getpixel((x, y))
                # Check if the pixel is not transparent
                if alpha != 0:
                    # Add the values to the total
                    total_red += red
                    total_green += green
                    total_blue += blue
                    total_pixels += 1

        # Calculate the average red, green, and blue values
        avg_red = total_red / total_pixels
        avg_green = total_green / total_pixels
        avg_blue = total_blue / total_pixels

        # Return the average color as a tuple of integers
        return colorsys.rgb_to_hsv(int(avg_red)/255, int(avg_green)/255, int(avg_blue)/255)


# Get the folder where the sprites are at
folder = input("Enter full path to the images folder: ")

# Loop through each image
for filename in os.listdir(folder):
    # The full path
    f = os.path.join(folder, filename)
    # If it's a file
    if (os.path.isfile(f)):
        # Get the average color
        color = average_color(f)
        # Get the relative path
        index = 0
        try:
            index = f.index("data")
        except:
            index = f.index("mods")
        relPath = f[index:].replace("\\", "/")

        # Prints out the formatted entry
        print("[\""+relPath+"\"] = {H="+str(round(color[0], 3))+",S=" +
              str(round(color[1], 3))+",V="+str(round(color[2], 3))+"},")
input("\nDONE")
