dofile_once("mods/inventory_sorter/settings.lua")
dofile_once("mods/inventory_sorter/utils.lua")

local sortGui = GuiCreate()
local player_id
local fullInventory
local slotsX
local slotsY
local inventoryComponent

actions_lookup = {}

function OnMagicNumbersAndWorldSeedInitialized()
    dofile_once("data/scripts/gun/gun.lua")
    if (settings.mod_compatibility) then
        GenerateLookup()
    else
        dofile_once("mods/inventory_sorter/actions_lookup.lua")
    end
end

function GenerateLookup()
    print("Generating lookup (inventory_sorter)...If this takes too long, disable mod_compatibility in settings.lua")
    for i, action in ipairs(actions) do
        if (string.find(action.name, "timer") or action.name == "$action_black_hole_death_trigger") then action.type = 2
        elseif (
            (string.find(action.name, "trigger") and not string.find(action.name, "add")) or
                action.name == "$action_summon_hollow_egg") then action.type = 1
        elseif (action.type > 0) then action.type = action.type + 2 end
        
        actions_lookup[action.id] = {
            name = action.name,
            description = action.description,
            sprite = action.sprite,
            mana = action.mana,
            price = action.price,
            max_uses = action.max_uses,
            type = action.type
        }
    end
end

function GetPlayerId()
    --If the player does not exist or changed id
    if (player_id == nil or EntityGetName(player_id) == nil) then
        local players = EntityGetWithTag("player_unit")
        if (#players == 0) then player_id = nil
        else
            player_id = players[1]
        end
    end
    return player_id
end

function GetInventories()
    if(player_id == nil) then return end

    --Gets the full inventory entity
    for i, child in ipairs(EntityGetAllChildren(player_id)) do
        if (EntityGetName(child) == "inventory_full") then
            fullInventory = child
        end
    end
    inventoryComponent = EntityGetFirstComponent(player_id, "Inventory2Component")
    return fullInventory, inventoryComponent
end

function OnPlayerSpawned(player_entity)
    player_id = player_entity
end

function OnWorldPreUpdate()
    GetPlayerId()
    --Draw the button only when the inventory is closed
    if (not GameIsInventoryOpen() and player_id) then
        local clicked = GuiImageButton(sortGui, 64209, settings.anchor, 20, "", "mods/inventory_sorter/files/ui/sort1.png")
        if (clicked and SortInventory()) then
            local x, y = EntityGetTransform(player_id)
            GamePlaySound("data/audio/Desktop/projectiles.bank", "ui/item_switch_places", x, y)
            GamePlaySound("data/audio/Desktop/projectiles.bank", "ui/button_click", x, y)
            GamePrint("Sorted inventory!")
        end
        GuiStartFrame(sortGui)
    end
end

function SortInventory()
    GetInventories()
    if(not fullInventory or not inventoryComponent) then return false end
    local items = EntityGetAllChildren(fullInventory)
    if (not items) then
        return false
    end

    --Sort the items
    for i = 2, #items do
        local j = i
        while j > 1 and Compare(items[j - 1], items[j], items) do --While the previous item needs to be moved forward
            --Swap the items
            items[j], items[j - 1] = items[j - 1], items[j]
            j = j - 1
        end
    end

    --Just in case the inventory changes size
    slotsX = ComponentGetValue2(inventoryComponent, "full_inventory_slots_x")
    slotsY = ComponentGetValue2(inventoryComponent, "full_inventory_slots_y")

    --Set the indexes of the items in the inventory
    local index = 0
    for i = 0, #items do
        if (items[i]) then
            SetIndexOfItem(items[i], index)
            index = index + 1
        end
    end
    ComponentSetValue2(inventoryComponent, "mForceRefresh", true)
    return true
end

function Compare(itemA, itemB, items)
    return settings.sort_by(itemA, itemB, items)
end

function SetIndexOfItem(item, index)
    local y = math.floor(index / slotsX)
    local x = index - y * slotsX

    local itemComponent = EntityGetComponentIncludingDisabled(item, "ItemComponent")
    ComponentSetValue2(itemComponent[1], "inventory_slot", x, y)
end
