dofile_once("mods/inventory_sorter/colors.lua")

--COMPARING--

function CompareAlphabetical(itemA, itemB)
    return GetTranslatedItemActionName(itemA) > GetTranslatedItemActionName(itemB)
end

function CompareTypeAlphabetical(itemA, itemB)
    local itemAType, itemBType = GetItemActionType(itemA), GetItemActionType(itemB)

    if (itemAType == itemBType) then return CompareAlphabetical(itemA, itemB)
    else return CompareType(itemA, itemB) end
end

function CompareType(itemA, itemB)
    local itemAType, itemBType = GetItemActionType(itemA), GetItemActionType(itemB)
    return itemAType > itemBType
end

function CompareTypeAmount(itemA, itemB, items)
    local itemAType, itemBType = GetItemActionType(itemA), GetItemActionType(itemB)

    if (itemAType == itemBType) then return CompareAmount(itemA, itemB, items)
    else return CompareTypeAlphabetical(itemA, itemB) end
end

function CompareAmount(itemA, itemB, items)
    local countA, countB = CountItemActionsWithSameName(itemA, items), CountItemActionsWithSameName(itemB, items)

    if (countA == countB) then return CompareTypeAlphabetical(itemA, itemB)
    else return countA < countB end
end

function CompareMana(itemA, itemB)
    local manaA, manaB = GetItemActionMana(itemA), GetItemActionMana(itemB)
    return manaA > manaB
end

function CompareTypeMana(itemA, itemB)
    local itemAType, itemBType = GetItemActionType(itemA), GetItemActionType(itemB)
    if(itemAType == itemBType) then return CompareMana(itemA, itemB)
    else return CompareTypeAlphabetical(itemA, itemB) end
end

function CompareColors(itemA, itemB)
    local actionA, actionB = GetAction(itemA), GetAction(itemB)

    if(not actionA or not actionB) then return nil end

    local colorA, colorB = hsv_color_lookup[actionA.sprite] , hsv_color_lookup[actionB.sprite]

    if(not colorA or not colorB) then return nil end

    if(colorA.H == colorB.H) then return colorA.S > colorB.S end
    return colorA.H > colorB.H
end

function CompareTypeColors(itemA, itemB)
    local itemAType, itemBType = GetItemActionType(itemA), GetItemActionType(itemB)

    if(itemAType == itemBType) then return CompareColors(itemA, itemB)
    else return CompareTypeAlphabetical(itemA, itemB) end
end

--OTHER--

local cacheLimit = 500
local cache = {}
local keyQueue = {}

function AddCache(key, value)
    cache[key] = value
    table.insert(keyQueue, key)
    if(#cache > cacheLimit) then
        local oldestKey = table.remove(keyQueue, 1)
        cache[oldestKey] = nil
        print("Deleting oldest value!")
    end

end

function GetCache(key)
    return cache[key]
end

--From the item entity, get the action_id
function GetActionId(item)
    local key = tostring(item).."_id"
    local action_id = GetCache(key)
    if(action_id) then return action_id end

    local actionComponent = EntityGetComponentIncludingDisabled(item, "ItemActionComponent")

    if (not actionComponent) then return nil end

    action_id = ComponentGetValue2(actionComponent[1], "action_id")
    AddCache(key, action_id)
    return action_id
end

function GetAction(item)
    local action = actions_lookup[GetActionId(item)]
    return action
end

--From the item entity, get the action type
function GetItemActionType(item)
    local action = GetAction(item)
    if (not action) then return 10 end
    return action.type
end

function GetItemActionMana(item)
    local action = GetAction(item)
    if (not action) then return 9999 end
    return action.mana
end

--From the item entity, get the action description
function GetItemActionDescription(item)
    local action = GetAction(item)
    if (action) then return action.description end
    return GameTextGetTranslatedOrNot(action.description)
end

function GetFileName(path)
    return path:match("^.+/(.+)$")
end

--From the item entity, get the translated action name
function GetTranslatedItemActionName(item)
    return GetTranslatedActionIdName(GetActionId(item))
end

--From the action_id, get the translated action name
function GetTranslatedActionIdName(action_id)
    local key = action_id.."_name"
    local name = GetCache(key)
    if(name) then return name end

    local action = actions_lookup[action_id]
    if (not action) then name = action_id:gsub("$action", ""):gsub("_", " ")
    else name = GameTextGetTranslatedOrNot(action.name) end
    AddCache(key, name)

    return name
end

function CountItemActionsWithSameName(item, items)
    local count = 0
    for i = 1, #items do
        if (GetTranslatedItemActionName(items[i]) == GetTranslatedItemActionName(item)) then count = count + 1 end
    end
    return count
end
